@@ -17,11 +17,13 @@
  *  add better screen-size sensing
  *  expand color use for PCs and introduce it for SVR3 UNIX machines
  *  extend '/' to multiple identifications
+ *  allow meta key to be used to invoke extended commands
  *  fix various minor bugs, and do further code cleaning
  */
 
 /*
  *  Patch 3, September 6, 1989
+ *  add war hammers and revise object prices
  *  extend prototypes to ANSI compilers in addition to the previous MSDOS ones
  *  move object-on-floor references into functions in preparation for planned
  *	data structures to allow faster access and better colors
@@ -29,4 +31,16 @@
  *	patches
  */
 
-#define PATCHLEVEL	3
+/*
+ *  Patch 4, September 27, 1989
+ *  add support for VMS (courtesy David Gentzel)
+ *  move monster-on-floor references into functions and implement the new
+ *	lookup structure for both objects and monsters
+ *  extend the definitions of objects and monsters to provide "living color"
+ *	in the dungeon, instead of a single monster color
+ *  ifdef varargs usage to satisfy ANSI compilers
+ *  standardize on the color 'gray'
+ *  assorted bug fixes
+ */
+
+#define PATCHLEVEL	4
