@@ -37,6 +37,9 @@ doversion(){
 #endif
 #ifdef UNIX
 		"Unix",
+#endif
+#ifdef VMS
+		"VMS",
 #endif
 		VERSION,
 #ifndef BETA
